<?php

if( ! defined( 'ABSPATH' ) ) die;

/**
 * Include semantic ui JS + CSS + Functions
 */

class WPSC_helpers {

	// use to avoid notices when accesing an array element
	static public function valArrElement($arr=null, $elem=null) {
		if (!is_array($arr)) return false;
		if (!$elem and $elem!=="0" and $elem!==0) return false;
		if (!array_key_exists($elem, $arr)) return false;
		return true;
	}

	// get network information from json file
	static function getNetworks() {
	    if (file_exists($json_file = dirname(dirname(__FILE__)).'/assets/json/networks.json') and 
	    	is_array( $arr = json_decode( file_get_contents($json_file), true ) ) ) {
	    	return $arr;
		} else {
			return false;
		}
	}

	static public function getFlavorColor($flavor) {
		if ($flavor=="vanilla") return "yellow";
		if ($flavor=="pistachio") return "olive";
		if ($flavor=="chocolate") return "brown";
		if ($flavor=="mango") return "orange";
		if ($flavor=="mochi") return "purple";
		if ($flavor=="suika") return "red";
		if ($flavor=="matcha") return "green";
		if ($flavor=="yuzu") return "yellow";
		if ($flavor=="ube") return "purple";
		if ($flavor=="almond") return "almond";
		if ($flavor=="ikasumi") return "black";
		if ($flavor=="azuki") return "brown";
	}

	// format a number with 18 decimals and the proper number separators
    static public function formatNumber($num) {

    	// convert to float
    	$num = floatval($num);

    	// validate number of decimals to show, use 4 as default
    	if (!is_numeric($ndts = WPSCSettingsPage::numberOfDecimalsToShow())) {
    		$ndts = 4;
    	}

    	// add thousands and decimals separators
        $nf = number_format($num, $ndts, WPSCSettingsPage::numberFormatDecimals(), WPSCSettingsPage::numberFormatThousands());

        return $nf;

    }

	// format a number with 18 decimals and the proper number separators
    static public function formatNumber2($num, $ndts) {

    	// convert to float
    	$num = floatval($num);

    	// add thousands and decimals separators
        $nf = number_format($num, $ndts, WPSCSettingsPage::numberFormatDecimals(), WPSCSettingsPage::numberFormatThousands());

        return $nf;

    }	

    // create a page with QR Scanner and NFT minter if not already created
    static public function createPluginPages() {

    	if (WPSC_assets::getQrScanner()===false) {
			$new_page_id = wp_insert_post( array(
	            'post_title'     => 'WPSC QR Scanner',
	            'post_type'      => 'page',
	            'comment_status' => 'closed',
	            'ping_status'    => 'closed',
	            'post_content'   => '[wpsc_qr_scanner]',
	            'post_status'    => 'publish',
	            'post_author'    => get_user_by( 'id', 1 )->user_id
	        ) );
			if ( $new_page_id && ! is_wp_error( $new_page_id ) ) {
	            update_post_meta( $new_page_id, 'wpsc_is_scanner', true );
	            update_post_meta( $new_page_id, '_wp_page_template', 'wpsc-clean-template.php' );
	        }
    	}

    	if (WPSC_assets::getNFTMintPage()===false) {
			$new_page_id = wp_insert_post( array(
	            'post_title'     => 'NFT Mint',
	            'post_type'      => 'page',
	            'comment_status' => 'closed',
	            'ping_status'    => 'closed',
	            'post_content'   => '[wpsc_nft_mint]',
	            'post_status'    => 'publish',
	            'post_author'    => get_user_by( 'id', 1 )->user_id
	        ) );
			if ( $new_page_id && ! is_wp_error( $new_page_id ) ) {
	            update_post_meta( $new_page_id, 'wpsc_is_nft_minter', true );
	            update_post_meta( $new_page_id, '_wp_page_template', 'wpsc-clean-template.php' );
	        }
    	}

    	if (WPSC_assets::getNFTMyItemsPage()===false) {
			$new_page_id = wp_insert_post( array(
	            'post_title'     => 'NFT My Items',
	            'post_type'      => 'page',
	            'comment_status' => 'closed',
	            'ping_status'    => 'closed',
	            'post_content'   => '[wpsc_nft_my_items]',
	            'post_status'    => 'publish',
	            'post_author'    => get_user_by( 'id', 1 )->user_id
	        ) );
			if ( $new_page_id && ! is_wp_error( $new_page_id ) ) {
	            update_post_meta( $new_page_id, 'wpsc_is_nft_my_items', true );
	            update_post_meta( $new_page_id, '_wp_page_template', 'wpsc-clean-template.php' );
	        }
    	}

    	if (WPSC_assets::getNFTMyGalleriesPage()===false) {
			$new_page_id = wp_insert_post( array(
	            'post_title'     => 'NFT My Galleries',
	            'post_type'      => 'page',
	            'comment_status' => 'closed',
	            'ping_status'    => 'closed',
	            'post_content'   => '[wpsc_nft_my_galleries]',
	            'post_status'    => 'publish',
	            'post_author'    => get_user_by( 'id', 1 )->user_id
	        ) );
			if ( $new_page_id && ! is_wp_error( $new_page_id ) ) {
	            update_post_meta( $new_page_id, 'wpsc_is_nft_my_galleries', true );
	            update_post_meta( $new_page_id, '_wp_page_template', 'wpsc-clean-template.php' );
	        }
    	}

    	if (WPSC_assets::getNFTAuthorsPage()===false) {
			$new_page_id = wp_insert_post( array(
	            'post_title'     => 'NFT Authors',
	            'post_type'      => 'page',
	            'comment_status' => 'closed',
	            'ping_status'    => 'closed',
	            'post_content'   => '[wpsc_nft_author]',
	            'post_status'    => 'publish',
	            'post_author'    => get_user_by( 'id', 1 )->user_id
	        ) );
			if ( $new_page_id && ! is_wp_error( $new_page_id ) ) {
	            update_post_meta( $new_page_id, 'wpsc_is_nft_author', true );
	            update_post_meta( $new_page_id, '_wp_page_template', 'wpsc-clean-template.php' );
	        }
    	}

		if (WPSC_assets::getNFTMyBidsPage()===false) {
			$new_page_id = wp_insert_post( array(
	            'post_title'     => 'NFT My Bids',
	            'post_type'      => 'page',
	            'comment_status' => 'closed',
	            'ping_status'    => 'closed',
	            'post_content'   => '[wpsc_nft_my_bids]',
	            'post_status'    => 'publish',
	            'post_author'    => get_user_by( 'id', 1 )->user_id
	        ) );
			if ( $new_page_id && ! is_wp_error( $new_page_id ) ) {
	            update_post_meta( $new_page_id, 'wpsc_is_nft_my_bids', true );
	            update_post_meta( $new_page_id, '_wp_page_template', 'wpsc-clean-template.php' );
	        }
    	}

    }

    // return the short version of an Ethereum address
	public static function shortify($address, $ultra=false) {
		if ($address) {
			if ($ultra) {
				return substr($address, 0, 4) . '...' . substr($address, -2);
			} else {
				return substr($address, 0, 6) . '...' . substr($address, -4);
			}	
		}
	}

	public static function languages() {
		load_plugin_textdomain( 'wp-smart-contracts', false, basename( dirname( __DIR__ ) ) . '/languages/' );
	}

	public static function renderWPICInfo() {
	    $m = new Mustache_Engine;
	    return $m->render(
	      WPSC_Mustache::getTemplate('metabox-wpic-info'), 
	      [
	        "title" => __('Scale Ethereum', 'wp-smart-contracts'),
			"logo" => dirname( plugin_dir_url( __FILE__ )) . '/assets/img/xdai.png',
	        "description_1" => __('Layer 2 Solutions supported by WPSmartContracts', 'wp-smart-contracts'),
	        "description_2" => __('Now you can deploy all smart contracts flavors in four blockchains.', 'wp-smart-contracts'),
	        "feature_1" => __('Ethereum', 'wp-smart-contracts'),
	        "feature_2" => __('xDai Chain', 'wp-smart-contracts'),
	        "feature_3" => __('Binance Smart Chain', 'wp-smart-contracts'),
	        "feature_4" => __('Polygon (previously Matic)', 'wp-smart-contracts'),
	        "feature_5" => __('Easy deploy with lower fees', 'wp-smart-contracts'),
			"blockchain_img" => dirname( plugin_dir_url( __FILE__ )) . '/assets/img/blocks.png',
	        "claim" => __('Claim 5,000 WPIC', 'wp-smart-contracts'),
	        "claim-note" => __('0.1 Ξ', 'wp-smart-contracts'),
	        "learn" => __('Learn more', 'wp-smart-contracts'),
	      ]
	    );
	}

	static public function nativeCoinName($net) {
		$networks = self::getNetworks();
		if (WPSC_helpers::valArrElement($networks, $net) and 
			WPSC_helpers::valArrElement($networks[$net], "coin-symbol") and
			$networks[$net]["coin-symbol"]) {
			return $networks[$net]["coin-symbol"];
		} else {
            return "Ether";
		}
	}

	static public function getIdFromShortcodes() {

		$the_id = get_the_ID();

		$data = get_post_field("post_content", $the_id);

		foreach(['wpsc_coin', 'wpsc_crowdfunding', 'wpsc_ico', 'wpsc_staking'] as $shortcode) {

			preg_match("/\[$shortcode (.+?)\]/", $data, $dat);
		
			$dat = array_pop($dat);
			$dat= explode(" ", $dat);
			$params = array();
			foreach ($dat as $d){
				if ($d) {
					list($opt, $val) = explode("=", $d);
					$params[$opt] = trim($val, '"');	
				}
			}
		
			if (WPSC_helpers::valArrElement($params, "id") and $params["id"]) {
				$the_id = $params["id"];
				break;
			}
		
		}
		
		return $the_id;
	}

	static public function flavors() {
		return ["vanilla", "pistachio", "chocolate", "mango", "raspberry", "bluemoon", "matcha", "mochi", "suika", "yuzu", "azuki", "ikasumi", "ube", "almond"];
	}

	static public function getNetworkInfoJSON($flavor) {

		$refresh_cache = false;
		$option_name = "wpsc-1-3-6";
		if (!get_option($option_name)) {
			$refresh_cache = true;
			update_option($option_name, true);
		}

		// store in transient for 6 hours
		$transient_name = WPSC_Endpoints::transientPrefix . "net_info_json";
		$json = false;
		if (!$refresh_cache and $t = get_transient($transient_name)) {
			$json = $t;
		} else {
			if (defined("WPSC_NETINFO_LOCAL_PATH")) {
				$response = wp_remote_get(WPSC_NETINFO_LOCAL_PATH);
			} else {
				$response = wp_remote_get("https://api.wpsmartcontracts.com/netinfo.json");
			}
			
			if ( is_array( $response ) && ! is_wp_error( $response ) ) {
				$json = $response['body'];
				if ($json) {
					json_decode($json);
					if (json_last_error() === JSON_ERROR_NONE) {
						set_transient($transient_name, $json, 21600);
					}
				}
			}
		}
		if ($json) {
			$json = json_decode($json, true);
			$json_ret = [];
			if (WPSC_helpers::valArrElement($json, "data")) {
				foreach($json["data"] as $i => $j) {
					if (WPSC_helpers::valArrElement($json["data"][$i], "discontinued")) continue;

					if (WPSC_helpers::valArrElement($json["data"][$i], $flavor) and $json["data"][$i][$flavor]) {
						if ($json["data"][$i]["type"]=="Mainnet") {
							$json["data"][$i]["is_mainnet"]=true;
						}
						if (WPSC_helpers::valArrElement($json["data"][$i], "blockchain")) {
							switch($json["data"][$i]["blockchain"]) {
							case "Ethereum":
								$json["data"][$i]["is_ethereum"]=true;
								break;
							case "Bitcoin":
								$json["data"][$i]["is_bitcoin"]=true;
								break;
							}		
						}
						$json["data"][$i]["show"]=true;
						if ($json["data"][$i][$flavor][0]=="Free") {
							$json["data"][$i]["free"]=true;
						} else {
							$json["data"][$i]["fee"]=$json["data"][$i][$flavor][0];
							$json["data"][$i]["fee_usd"]=$json["data"][$i][$flavor][1];
						}
						$json_ret[] = $json["data"][$i];
					}
				}
				return ["date_time" => $json["date_time"], "data"=>$json_ret, "asset_path"=>plugin_dir_url( dirname( __FILE__ ) ) . 'assets/img/'];    
			}
	   }

	}

	static function userGalleriesMessage() {
		if (WPSC_helpers::valArrElement($_COOKIE, "wpscGalleriesMessageDismissed") and $_COOKIE["wpscGalleriesMessageDismissed"]) {
			return '';
		} else {
			return '<div id="wpsc-galleries-message" class="notice notice-info is-dismissible"><p> If you want to assign galleries to users, go to the <a href="' . admin_url("users.php") . '">users section</a>, edit the user\'s profile and assign galleries accordingly.</a></p></div>';
		}
	}
	
	static function getCollectionID() {

		$collection_id = 0;
		if (WPSC_helpers::valArrElement($_GET, 'id')) {
            $collection_id = (int) $_GET["id"];
			if ($collection_id and get_post_type($collection_id)=="nft-collection") {
				return $collection_id;
			}
		}

		$post_id = get_the_ID();
		$post_type = get_post_type($post_id);

		if ($post_type == "nft-collection") {
			$collection_id = $post_id;
		} elseif($post_type=="nft") {
			$collection_id = get_post_meta($post_id, "wpsc_item_collection", true);
        }

		return $collection_id;

	}

	static function getGalleryIdCookie($collection_id) {
		
		$wpsc_sub_collections = get_post_meta($collection_id, 'wpsc_sub_collections', true);
        if (!$wpsc_sub_collections) return false;

		if (WPSC_helpers::valArrElement($_GET, "gal_id")) {
			$gal_id = (int) $_GET["gal_id"];
			if ($gal_id) {
				return $gal_id;
			}
		}

		$cookie_name = "gal_id_" . $collection_id;
		if (WPSC_helpers::valArrElement($_COOKIE, $cookie_name)) {
			return $_COOKIE[$cookie_name];
		} else {
			return false;
		}
	}

    static function processGalleryCookies() {

		$collection_id = self::getCollectionID();

		if (!$collection_id) return;

		// If we get the gallery id parameter
        if (self::valArrElement($_GET, 'gal_id')) {
			$cookie_name = "gal_id_" . $collection_id;
            // process reset
            if ($_GET['gal_id']=="reset") {
				// unset cookie
                if (WPSC_helpers::valArrElement($_COOKIE, $cookie_name)) {
					unset($_COOKIE[$cookie_name]);
                }
                setcookie($cookie_name, '', time() - 3600, '/');
            // process numeric gallery id
            } else {
                $current_gal_id = (int) $_GET["gal_id"];
                // store it in cookies
                if ($current_gal_id) {
                    setcookie($cookie_name, $current_gal_id, time()+3600, '/');
                }
            }
        }

	}

	static public function walkQuery(&$item, $key, $params) {
		
		$term_id_int = (int) $item["term_id"];

		if (!$term_id_int) return;

		$url = get_term_link($term_id_int, "nft-gallery");
		
		if (is_wp_error($url)) return;

        $url = add_query_arg("gal_id", $item["term_id"], $url);
        $url = add_query_arg("id", $params["collection_id"], $url);

		$item["url"] = $url;

        if ($params["current_gal_id"] == $item["term_id"]) {
            $item["selected"] = true;
        }
    }

	static public function actualURL() {
        return (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? "https" : "http") . "://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]";
	}

	static public function currentGalleryLink($collection_id) {

        $url = remove_query_arg("gal_id", self::actualURL());

        $current_gal_id = 0;

        // If we get the gallery id parameter
        if (self::valArrElement($_GET, 'gal_id')) {
            $current_gal_id = (int) $_GET["gal_id"];
        }

        $cookie_name = 'gal_id_'.$collection_id;

        // try to get the gallery id from cookies if it is not passed as get parameter
        if (!$current_gal_id and self::valArrElement($_COOKIE, $cookie_name)) {
            $current_gal_id = (int) $_COOKIE[$cookie_name];
        }

		if (!$current_gal_id) {
			return add_query_arg("gal_id", "reset", get_permalink($collection_id));
		} else {
			$url = get_term_link($current_gal_id, "nft-gallery");		
			if (is_wp_error($url)) return;
			$url = add_query_arg("gal_id", $current_gal_id, $url);
			$url = add_query_arg("id", $collection_id, $url);
			return $url;
		}

    }

	static public function getGalleriesOfCollection($collection_id) {

        $url = remove_query_arg("gal_id", self::actualURL());

        $current_gal_id = 0;

        // If we get the gallery id parameter
        if (self::valArrElement($_GET, 'gal_id')) {
            $current_gal_id = (int) $_GET["gal_id"];
        }

        $cookie_name = 'gal_id_'.$collection_id;

        // try to get the gallery id from cookies if it is not passed as get parameter
        if (!$current_gal_id and self::valArrElement($_COOKIE, $cookie_name)) {
            $current_gal_id = (int) $_COOKIE[$cookie_name];
        }

		$res = WPSC_Queries::queryGalleriesOfCollection($collection_id);

        array_walk($res, ["WPSC_helpers", "walkQuery"], ["url"=>$url, "current_gal_id"=>$current_gal_id, "collection_id"=>$collection_id]);

        $first = ["term_id"=>0, "name" => "All Galleries", "url" => add_query_arg("gal_id", "reset", get_permalink($collection_id))];
        if (!$current_gal_id) {
            $first["selected"]=true;
        }
        array_unshift($res, $first);
        
        return $res;

    }

	public static function updatePostMeta($field, $field_meta, $arr, $post_id) {
		if (WPSC_helpers::valArrElement($arr, $field)) {
			$val = WPSC_Metabox::cleanUpText($arr[$field]);
			update_post_meta($post_id, $field_meta, $val);
			return $val;
		} else {
			update_post_meta($post_id, $field_meta, null);
		}
	}	

}

/**
 * Warnings in wp-admin
 */

add_action('admin_notices', function () {

	// check Infura settings

	$options = get_option('etherscan_api_key_option');
	$infura_api_key = (WPSC_helpers::valArrElement($options, "infura_api_key") and !empty($options["infura_api_key"]))?$options["infura_api_key"]:false;

	if (empty(trim($infura_api_key))) {
		echo '<div class="notice notice-error is-dismissible">
			<h3>WP Smart Contracts alert - Action needed!</h3><p>To use the WP Smart Contracts plugin properly you need to setup a free <a href="https://infura.io/" target="_blank">Infura Project ID</a> in <a href="'. get_admin_url() . 'options-general.php?page=etherscan-api-key-setting-admin">WP Smart Contracts Settings</a> in your WordPress install</p><p>Otherwise your site may not allow users to properly interact with your Smart Contract if they do not have Metamask installed</p>
		</div>';
	}

	// check that rest api is alive

	$endpoint = get_rest_url(null, 'wpsc/v1/ping');

	$response = wp_remote_get( $endpoint, null );

	if (!WPSC_helpers::valArrElement($response, "response") or !WPSC_helpers::valArrElement($response["response"], "code") or $response["response"]["code"] != "200") {
		
		$get_rest_url = get_rest_url(null, '');

		echo <<<HELP
		<div class="notice notice-error is-dismissible">
			<h3>WP Smart Contracts alert - Action needed!</h3>
			<p>Looks like the <a href="https://developer.wordpress.org/rest-api/" target="_blank">WP Rest API</a> is not working properly in your WordPress installation</p>
			<p>Please check that the URL: <a href="$get_rest_url" target="_blank">$get_rest_url</a> is not failing</p>
			<p>For more information please visit: <a href="https://wordpress.org/support/topic/404-error-on-coin-page-already-created-the-coin-on-mainnet/" target="_blank">our support page</a></p>
		</div>
HELP;
	}

	// NEWS

	if (!WPSC_helpers::valArrElement($_COOKIE, "notice_hide_ikasumi_azuki")) {

		$wpsc_logo = dirname( plugin_dir_url( __FILE__ )) . '/assets/img/wpsc-logo.png';
		$wc_logo = dirname( plugin_dir_url( __FILE__ )) . '/assets/img/ikasumi-azuki.png';
	
		echo <<<BLOCKCHAIN
		<div id="wpic-notification" class="notice notice-info is-dismissible">

		<img src="$wpsc_logo" style="width: 300px;">

		<hr>

		<h1>Hello!, thanks for installing WP Smart Contracts v.1.3.9!</h1>

		<p><img src="$wc_logo"></p>

		<h2>= What's new in 1.3.9 =</h2>
		» New Smart Contracts: Ikasumi and Azuki<br>
		» ERC1155 Marketplace support<br>
		» New optimized Yuzu contract<br>
		» Yuzu burn capability removed<br>
		» Bug fix in auctions in skin 1<br>
		» Bug fix adding tokens to metamask from NFT UI<br>
		» Bug fix showing network of NFTs in Backend<br>
		» Bug fix in All Galleries view<br>
		
		<hr>
		
		<p style="color: #777;"><input type="checkbox" id="wpic-no-show"> Got it, Thanks!. Please don't show this again.</p>
		<button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button>
	 </div>
BLOCKCHAIN;
	}

});

/**
 * Load i18n files
 */
add_action( 'plugins_loaded', ['WPSC_helpers', 'languages'] );


/**
 * One time notification
 */
class OneTimeNotifications {

    public static $_notices  = array();

    public function __construct() {
		add_action( 'shutdown', array( $this, 'save_errors' ) );
    }

    public static function add_notice( $title, $text ) {
        self::$_notices[] = [$title, $text];
    }

    public function save_errors() {
        update_option( 'wpsc_custom_notices', self::$_notices );
    }

    public function output_errors() {
        $errors = maybe_unserialize( get_option( 'wpsc_hide_custom_notices' ) );

        if ( ! empty( $errors ) ) {

            echo '<div id="mc_notices" class="error notice is-dismissible">';

			echo '<h3>WP Smart Contracts change log</h3>';

            foreach ( $errors as $error ) {
				if ($error[0]) echo '<p><b>&raquo; ' . wp_kses_post( $error[0] ) . '</b>';
                if ($error[1]) echo '<br>' . wp_kses_post( $error[1] ) . '</p>';
            }

            echo '</div>';

			update_option( 'wpsc_hide_custom_notices', 1 );

		}
    }

}

if (!get_option( 'wpsc_onetime_notice_1' )) {
	add_action( 'shutdown', function () {
		echo '<div class="notice notice-info is-dismissible">';
		echo '<h3>WP Smart Contracts change log</h3>';
		foreach ([
			["We ♥ WP5.5", "Now WPSmartContracts is compatible with WordPress 5.5"],
			["Uniswap is in da house!", "Now you can add your token to Uniswap and include it in the Exchanges section"],
			["Because good ideas needs your support!", "Starting January 1, 2021 deploy fees may apply, visit <a href=\"https://wpsmartcontracts.com\" target=\"_blank\">wpsmartcontracts.com</a> for more information"]
		] as $notice ) {
			echo '<p><b>&raquo; ' . $notice[0] . '</b>';
			echo '<br>' . $notice[1] . '</p>';
		}
		echo '</div>';
		update_option( 'wpsc_onetime_notice_1', 1 );
	});
}
